// ////////////////////////////////////////////////////
//
// StdView.cpp : implementation of the CStdView class
// Based on class COGLView : public CView
// Inspect COGLView for standard OpenGL initialization
//
////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"
#include "math.h"

#include "StdDoc.h"
#include "StdView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////
// You can put definitions, variables and helperfunctions
// here. Functions that does not rely on CStdView member
// variables or functions

//////////////////////////////////////////////////////////
// CStdView

IMPLEMENT_DYNCREATE(CStdView, COGLView)

BEGIN_MESSAGE_MAP(CStdView, COGLView)
	//{{AFX_MSG_MAP(CStdView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////
// CStdView construction/destruction

CStdView::CStdView()
{	// initialize angles, see DrawScene
	m_xv=0.0f;
	m_yv=0.0f;
	m_hRC=NULL;
	m_hDC=NULL;
};

CStdView::~CStdView()
{}

////////////////////////////////////////////////////////////
// CStdView diagnostics
// You should usually not touch these:
#ifdef _DEBUG
void CStdView::AssertValid() const
{COGLView::AssertValid();}
void CStdView::Dump(CDumpContext& dc) const
{COGLView::Dump(dc);}

CStdDoc* CStdView::GetDocument() // non-debug version is inline
{	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CStdDoc)));
	return (CStdDoc*)m_pDocument;
}
#endif //_DEBUG

//////////////////////////////////////////////////////////////
// CStdView operations

void CStdView::InitializeRContext()
{
   glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);
    
	// prepare ligthsource
	GLfloat ambient[] = {0.2f,0.2f,0.2f,1.0f };
    GLfloat diffuse[] = {1.0f,1.0f,1.0f,1.0f };
    GLfloat position[] = {20.0f,30.0f,10.0f,0.0f };
    GLfloat lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };


    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
	
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	
	// smooth the drawing
	glShadeModel(GL_SMOOTH);

	// nomalize all normals
	glEnable(GL_NORMALIZE);

    // set background to light gray
	glClearColor(0.8f, 0.8f, 0.8f, 0.0f);
}


void CStdView::SetPerspective(int cx, int cy)
{
    // We will work on the projection matrix,
	// so set matrix mode
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	// set up your viewing volume
	gluPerspective(70.0f, (GLdouble)cx/cy, 0.1f, 70000.0f);
	// reset matrix mode 
	glMatrixMode(GL_MODELVIEW);
}


void CStdView::DrawScene()
{
	// Clear the color and depth buffers.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
	// Set up for scene
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// position the eye relative to scene
	gluLookAt(	400.0f,400.0f,400.0f,	// eye
				0.0f,0.0f,300.0f,		// looking at
				0.0f,0.0f,1.0f);		// is up
	
	// rotate around x-axis 
	glRotatef(m_xv,1.0f,0.0f,0.0f);
	// rotate around z-axis 
	glRotatef(m_yv,0.0f,0.0f,1.0f);

	
	DrawStairs(	100.0f, // outer radius
				15.0f,  // inner radius
				40.0f,  // step height
				5.0f,	// step material height
				30.0f,	// stepangle
				15);	// step count
							
	// finnish drawing
	glFlush();
}

void CStdView::DrawStairs(	float R, // outer radius
							float r, // inner radius
							float H, // element height
							float h, // stairwing thickness 
							float v, // stairwing fanangle
							int N)	 // element count
{
		
	// assuming GL_NORMALIZE is enabled
	// Degrees to Radians (supposing the angle v is given in degrees)
	float DtoR=3.1415f/180.0f;
	// will place the stairs with cylinder in origo
	// extending along z.

	// interpret r as the outer radius of
	// a stairs element. Assuming that the
	// material in the elements ring is d thick
	// assume d=h 
	float d=h;


	// will need a quadric
	GLUquadricObj *glpQ=gluNewQuadric();
	
	//----- draw the basic cylinder, in case it should extend
	// beyond the elements
	CMaterial::SetMaterial(EMERALD);
	gluCylinder(glpQ,r-d,r-d,N*H,20,20);


	// ----- make the elements
	// change materiale to show the difference
	CMaterial::SetMaterial(BRONZE);

	// strategy is:
	// 1 Draw all the cylindrical parts
	// For each step:
	// a draw the botton fan
	// b draw top fan
	// c draw the edge


	// 1 make all the elements cylindrical parts in a first loop
	glPushMatrix();
		for(int cix=0;cix<N;cix++)
		{
			gluCylinder(glpQ,r,r,H,20,20);
			glTranslatef(0.0f,0.0f,H);
		}
	glPopMatrix();

	// translate so we get in position to draw
	// the stairwings in the middle of each staircylinder
	glTranslatef(0.0f,0.0f,H/2.0f-h/2.0f);
	
	// need an angle to step along a stairwing
	float smallv=0.0f;

	// assuming the stairs are aligned

	for(int ix=0;ix<N;ix++) // each element
	{
		glPushMatrix();
		// a stairwing is a circlesegment
		// a bottom
		glBegin(GL_TRIANGLE_FAN);//GL_POLYGON is an alternative
			glNormal3f(0.0f,0.0f,-1.0f); // downwards since it is under
			glVertex3f(0.0f,0.0f,0.0f);
			smallv=0.0f;
			while(smallv<v)
			{
				glVertex3f(R*(float)sin(DtoR*smallv),R*(float)cos(DtoR*smallv),0.0f);
				smallv+=0.5f;
			}
			glVertex3f(R*(float)sin(DtoR*v),R*(float)cos(DtoR*v),0.0f);
			glVertex3f(0.0f,0.0f,0.0f);
		glEnd();
		
		// b top is exactly the same exept all z-koordinates
		glTranslatef(0.0f,0.0f,h);
		glBegin(GL_TRIANGLE_FAN);//GL_POLYGON is an alternative
			glNormal3f(0.0f,0.0f,1.0f);// up since it is over
			glVertex3f(0.0f,0.0f,0.0f);
			smallv=0.0f;
			while(smallv<v)
			{
				glVertex3f(R*(float)sin(DtoR*smallv),R*(float)cos(DtoR*smallv),0.0f);
				smallv+=0.5f;
			}
			glVertex3f(R*(float)sin(DtoR*v),R*(float)cos(DtoR*v),0.0f);
			glVertex3f(0.0f,0.0f,0.0f);
		glEnd();
		glPopMatrix();

		// c edge - up and down
		glBegin(GL_TRIANGLE_STRIP);
			glNormal3f(1.0f,0.0f,0.0f);
			glVertex3f(0.0f,0.0f,0.0f);
			glVertex3f(0.0f,0.0f,h);
			smallv=0.0f;
			while(smallv<v)
			{
				glNormal3f(R*(float)sin(DtoR*smallv),R*(float)cos(DtoR*smallv),0.0f);
				glVertex3f(R*(float)sin(DtoR*smallv),R*(float)cos(DtoR*smallv),0.0f);
				glVertex3f(R*(float)sin(DtoR*smallv),R*(float)cos(DtoR*smallv),h);
				smallv+=0.5f;
			}
			glNormal3f(R*(float)sin(DtoR*v),R*(float)cos(DtoR*v),0.0f);
			glVertex3f(R*(float)sin(DtoR*v),R*(float)cos(DtoR*v),0.0f);
			glVertex3f(R*(float)sin(DtoR*v),R*(float)cos(DtoR*v),h);
			glNormal3f(1.0f,0.0f,0.0f);
			glVertex3f(0.0f,0.0f,0.0f);
			glVertex3f(0.0f,0.0f,h);
		glEnd();

		
		// rotate to get in position for next step
		glRotatef(v,0.0f,0.0f,1.0f);
		// elevate for next step
		glTranslatef(0.0f,0.0f,H);
	}


	// delete the quadric
	gluDeleteQuadric(glpQ);

}

void CStdView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// User has pressed left button.Make a note
	// of this and prepare for intuitive rotation of scene
	m_PLast=point;
	SetCapture();
	CView::OnLButtonDown(nFlags, point);
}

void CStdView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// free capture and terminate rotation
	ReleaseCapture();
	CView::OnLButtonUp(nFlags, point);
}

void CStdView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// if left button is down 
	if(nFlags && MK_LBUTTON)
	{	
		// and has moved a significant dist
		if((abs(m_PLast.x-point.x)>2)||(abs(m_PLast.y-point.y)>2))
		{
			// then we find an intuitive rot angle
			CRect R;
			GetClientRect(&R);
			
			m_yv+=90.0f*((GLfloat (point.x-m_PLast.x))/GLfloat (R.Width()));
			m_xv+=90.0f*((GLfloat (point.y-m_PLast.y))/GLfloat (R.Height()));
			
			// rememeber point and force immediate redraw 
			m_PLast=point;
			RedrawWindow();
		}
	}
}
